import React, {Component} from 'react'
import PropTypes from 'prop-types'
import { is, fromJS } from 'immutable'
import { Spin, Empty, notification, Carousel, Modal, Checkbox, Button } from 'antd'
import { LeftOutlined, RightOutlined } from '@ant-design/icons'

import Api from '@/api'
import UtilsDM from '@/utils/utils-datamanage.js'
import MKEmitter from '@/utils/events.js'
import asyncComponent from '@/utils/asyncComponent'
import TimerTask from '@/utils/timer-task.js'
import './index.scss'

const CardItem = asyncComponent(() => import('../cardItem'))
const NormalHeader = asyncComponent(() => import('@/tabviews/custom/components/share/normalheader'))

class DataCard extends Component {
  static propTpyes = {
    data: PropTypes.array,           // 统一查询数据
    config: PropTypes.object,        // 组件配置信息
    mainSearch: PropTypes.any,       // 外层搜索条件
  }

  state = {
    BID: '',                   // 上级ID
    config: null,              // 图表配置信息
    loading: false,            // 数据加载状态
    sync: false,               // 是否统一请求数据
    card: null,                // 卡片设置
    data: null,                // 数据
    BData: null,
    visible: false
  }

  loaded = false

  UNSAFE_componentWillMount () {
    const { data, initdata } = this.props
    let _config = fromJS(this.props.config).toJS()
    let _card = _config.subcards[0]

    let _data = null
    let _sync = _config.setting.sync === 'true'

    let BID = ''
    let BData = ''

    if (_config.setting.supModule) {
      BData = window.GLOB.CacheData.get(_config.setting.supModule)
    } else {
      BData = window.GLOB.CacheData.get(_config.$pageId)
    }
    if (BData) {
      BID = BData.$BID || ''
    }

    if (_config.setting.sync === 'true' && data) {
      _data = data[_config.dataName] || []
      _sync = false
    } else if (_config.setting.sync === 'true' && initdata) {
      _data = initdata || []
      _sync = false
    }

    if (_data) {
      this.loaded = true
      _data = _data.map((item, index) => {
        item.key = index
        item.$$uuid = item[_config.setting.primaryKey] || ''
        item.$$BID = BID || ''
        item.$$BData = BData || ''
        item.$Index = index + 1
        return item
      })
    }

    if (_card.setting.click) {
      _card.style.cursor = 'pointer'
    }

    if (!_config.wrap.height) { // 兼容
      _config.wrap.height = _config.style.height || '300px'
      delete _config.style.height
    }

    _card.style.height = _config.wrap.height

    _config.wrap.speed = (_config.wrap.speed || 3) * 1000

    this.setState({
      sync: _sync,
      data: _data,
      BID: BID || '',
      BData: BData || '',
      config: _config,
      card: _card,
      arr_field: _config.columns.map(col => col.field).join(','),
    }, () => {
      if (_config.setting.sync !== 'true' && _config.setting.onload === 'true') {
        setTimeout(() => {
          this.loadData()
        }, _config.setting.delay || 0)
      } else {
        this.openModal()
      }
    })
  }

  componentDidMount () {
    const { config } = this.state

    MKEmitter.addListener('resetSelectLine', this.resetParentParam)
    MKEmitter.addListener('queryModuleParam', this.queryModuleParam)
    MKEmitter.addListener('refreshByButtonResult', this.refreshByButtonResult)

    if (config.timer) {
      this.timer = new TimerTask()
      this.timer.init(config.uuid, config.timer, config.timerRepeats, () => {
        this.loadData('timer')
      })
    }

    if (config.$cache && !this.loaded) {
      Api.getLCacheConfig(config.uuid).then(res => {
        if (!res || this.loaded) return

        let _data = res.map((item, index) => {
          item.key = index
          item.$$uuid = item[config.setting.primaryKey] || ''
          item.$Index = index + 1 + ''
          return item
        })

        this.setState({data: _data})
      })
    }
  }

  shouldComponentUpdate (nextProps, nextState) {
    return !is(fromJS(this.state), fromJS(nextState))
  }

  UNSAFE_componentWillReceiveProps (nextProps) {
    const { sync, config, BID, BData } = this.state

    if (sync && !is(fromJS(this.props.data), fromJS(nextProps.data))) {
      let _data = []
      if (nextProps.data && nextProps.data[config.dataName]) {
        _data = nextProps.data[config.dataName] || []
      }

      _data = _data.map((item, index) => {
        item.key = index
        item.$$uuid = item[config.setting.primaryKey] || ''
        item.$$BID = BID || ''
        item.$$BData = BData || ''
        item.$Index = index + 1
        return item
      })

      this.loaded = true

      this.setState({sync: false, data: _data}, () => {
        this.openModal()
      })
    } else if (config.setting.useMSearch && nextProps.mainSearch && !is(fromJS(this.props.mainSearch), fromJS(nextProps.mainSearch))) {
      this.setState({}, () => {
        this.loadData()
      })
    }
  }

  componentWillUnmount () {
    this.setState = () => {
      return
    }
    MKEmitter.removeListener('resetSelectLine', this.resetParentParam)
    MKEmitter.removeListener('queryModuleParam', this.queryModuleParam)
    MKEmitter.removeListener('refreshByButtonResult', this.refreshByButtonResult)

    this.timer && this.timer.stop()
  }

  openModal = (ErrCode) => {
    const { config, data } = this.state
    
    if (config.wrap.display !== 'modal' || !data || data.length === 0) return

    let code = config.wrap.code || ('modal' + config.uuid)
    let tip = localStorage.getItem(code)

    if (!data[0].$$uuid || tip === data[0].$$uuid + data.length) return

    if (config.wrap.modalContent === 'update') {
      localStorage.setItem(code, data[0].$$uuid + data.length)

      Api.getAppVersion(true)

      if (ErrCode === '-1') {
        return
      }
    }

    setTimeout(() => {
      this.setState({visible: true})
    }, 200)
  }

  /**
   * @description 按钮执行完成后页面刷新
   * @param {*} menuId     // 菜单Id
   * @param {*} position   // 刷新位置
   * @param {*} btn        // 执行的按钮
   */
  refreshByButtonResult = (menuId, position, btn) => {
    const { config, BID } = this.state

    if (config.uuid !== menuId) return

    if ((position === 'mainline' || position === 'popclose') && config.setting.supModule && BID) { // 刷新源组件时，附带刷新上级行与当前组件
      MKEmitter.emit('reloadData', config.setting.supModule, BID)
    } else {
      this.loadData()
    }

    if (position === 'popclose') { // 执行启动弹窗的按钮所选择的刷新项
      btn.$tabId && MKEmitter.emit('refreshPopButton', btn.$tabId)
    }
  }

  resetParentParam = (MenuID, id, data) => {
    const { config } = this.state

    if (!config.setting.supModule || config.setting.supModule !== MenuID) return
    if (id !== this.state.BID || id !== '') {
      this.setState({ BID: id, BData: data }, () => {
        this.loadData()
      })
    }
  }

  /**
   * @description 导出Excel时，获取页面搜索排序等参数
   */
  queryModuleParam = (menuId, callback) => {
    const { mainSearch } = this.props
    const { arr_field, config } = this.state

    if (config.uuid !== menuId) return

    let searches = config.setting.useMSearch && mainSearch ? fromJS(mainSearch).toJS() : []

    callback({
      arr_field: arr_field,
      orderBy: config.setting.order || '',
      search: searches,
      menuName: config.name
    })
  }

  async loadData (type) {
    const { mainSearch } = this.props
    const { config, arr_field, BID, BData } = this.state

    if (config.setting.supModule && !BID) { // BID 不存在时，不做查询
      this.setState({
        data: [],
        loading: false
      })

      this.loaded = true
      return
    }

    let searches = config.setting.useMSearch && mainSearch ? fromJS(mainSearch).toJS() : []
    let requireFields = searches.filter(item => item.required && item.value === '')
    if (requireFields.length > 0) {
      return
    }

    if (type !== 'timer') {
      this.setState({
        loading: true
      })
    }

    let _orderBy = config.setting.order || ''
    let param = UtilsDM.getQueryDataParams(config.setting, arr_field, searches, _orderBy, '', '', BID)

    let result = await Api.genericInterface(param)
    if (result.status) {
      this.loaded = true
      if (config.$cache && config.setting.onload !== 'false') {
        Api.writeCacheConfig(config.uuid, result.data || '')
      }

      let data = result.data.map((item, index) => {
        item.key = index
        item.$$uuid = item[config.setting.primaryKey] || ''
        item.$$BID = BID || ''
        item.$$BData = BData || ''
        item.$Index = index + 1
        return item
      })

      this.setState({
        data: data,
        loading: false
      }, () => {
        this.openModal(result.ErrCode)
      })

      if (config.timer && config.clearField && result.data && result.data[0]) {
        let vals = (config.clearValue || '').split(',')
        if (vals.includes(result.data[0][config.clearField])) {
          this.timer && this.timer.stop()
        }
      }
    } else {
      this.setState({
        loading: false
      })
      this.timer && this.timer.stop()
      
      if (result.ErrCode === 'N') {
        Modal.error({
          title: result.message,
        })
      } else {
        notification.error({
          top: 92,
          message: result.message,
          duration: 10
        })
      }
    }
  }

  onTipChange = (e) => {
    const { config, data } = this.state

    let code = config.wrap.code || ('modal' + config.uuid)
    if (e.target.checked) {
      localStorage.setItem(code, data[0].$$uuid + data.length)
    } else {
      localStorage.removeItem(code)
    }
  }

  render() {
    const { config, loading, data, card, visible } = this.state

    if (config.wrap.empty === 'hidden' && (!data || data.length === 0)) return null
    
    if (config.wrap.display === 'modal') {
      return <Modal
        wrapClassName='mk-carousel-modal'
        visible={visible}
        maskClosable={false}
        width={config.wrap.modalWidth}
        onCancel={() => this.setState({visible: false})}
        footer={null}
        destroyOnClose
      >
        {visible ? <div className="custom-data-carousel-box" style={config.style}>
          <NormalHeader config={config}/>
          <div className="carousel-wrap" style={{height: config.wrap.height}}>
            {data.length > 1 && config.wrap.autoplay === 'false' ? <div className="prev" onClick={() => this.node && this.node.prev()}><LeftOutlined /></div> : null}
            {data.length > 1 && config.wrap.autoplay === 'false' ? <div className="next" onClick={() => this.node && this.node.next()}><RightOutlined /></div> : null}
            <Carousel
              autoplay={config.wrap.autoplay !== 'false'}
              dots={config.wrap.dots !== 'false'}
              dotPosition={config.wrap.dotPosition || 'bottom'}
              effect={config.wrap.effect || 'scrollx'}
              autoplaySpeed={config.wrap.speed}
              ref={ref => this.node = ref}
            >
              {data.map((item, index) => (
                <div key={index}>
                  <CardItem card={card} cards={config} data={item}/>
                </div>
              ))}
            </Carousel>
          </div>
        </div> : null}
        {config.wrap.modalContent !== 'update' ? <div className="mk-footer">
          <Checkbox defaultChecked={false} onChange={this.onTipChange}>不在提醒</Checkbox>
          <Button className="close" onClick={() => this.setState({visible: false})}>立即体验</Button>
        </div> : <div className="mk-footer">
          <Button className="close" onClick={() => this.setState({visible: false})}>知道了</Button>
        </div>}
      </Modal>
    }

    return (
      <div className="custom-data-carousel-box" id={'anchor' + config.uuid} style={config.style}>
        {loading ?
          <div className="loading-mask">
            {data ? <div className="ant-spin-blur"></div> : null}
            <Spin />
          </div> : null
        }
        <NormalHeader config={config}/>
        <div className="carousel-wrap" style={{height: config.wrap.height}}>
          {data && data.length > 0 ? <Carousel
            autoplay={config.wrap.autoplay !== 'false'}
            dots={config.wrap.dots !== 'false'}
            dotPosition={config.wrap.dotPosition || 'bottom'}
            effect={config.wrap.effect || 'scrollx'}
            autoplaySpeed={config.wrap.speed}
          >
            {data.map((item, index) => (
              <div key={index}>
                <CardItem card={card} cards={config} data={item}/>
              </div>
            ))}
          </Carousel> : <Empty description={false}/>}
        </div>
      </div>
    )
  }
}

export default DataCard